#ifndef ROUTER_H_
#define ROUTER_H_

#include <cgutils/exec.h>

#ifdef _WIN32
#else
	#include <sys/types.h>
	#include <sys/wait.h>
	#include <unistd.h>
	#include <errno.h>
	#include <sys/socket.h>
	#include <netdb.h>
	#include <netinet/tcp.h>
#endif

using namespace cgutils;

const cgutils::TStrStrMap routerIni = create_map<std::string, cgutils::TStrMap>
(
	"P2mqrouter",
	create_map<std::string, std::string>
	("threads", "5")
	("threads_concur", "3")
	("port", "{port}")
	("AS", "NS")
)
(
	"P2MQCRYPTO",
	create_map<std::string, std::string>
	("VERYSIMPLE", "P2MQRouterAsVerySimple")
 )
 (
	"AS:NS",
	create_map<std::string, std::string>
	("USERNAME", "{username}")
	("PASSWORD", "12345")
 )
 (
 "ROUTEINFO",
 create_map<std::string, std::string>
 /*("type", "{routertype}")
 ("default", "{routedefault}")*/
 ("type", "server")
 ("default", "172.27.129.105:3001")
 )
 (
 "p2syslog",
 create_map<std::string, std::string>
 ("logfile", "{logfile}")
 ("logfileperday", "0")
 ("logfiledepth", "")
 ("logtoconsole", "0")
 ("logfilecache", "")
 ("traceini", "{tracefile}")
 )
 (
 "grnt",
 create_map<std::string, std::string>
 ("dbconnect", "P2DBSQLite3;;:memory:")
 )
 ;

class Router
{
	public:
		int port;

		Router(int port) : port(port) {}

		~Router()
		{
			stop();
		}

		int check();
		int start(const std::string& iniFile, const std::string pathExe = "");
		int start(const std::string& pathExe, TStrMap& valueMap, int& port);
		int startDirect(const std::string& pathExe, TStrMap& valueMap, int& port);
		int start(const std::string& pathExe, TStrMap& valueMap, int& port, const cgutils::TStrStrMap& ini);
		int kill(bool hard = false)
		{
			if (hard)
				return proc.kill();
			else
				return proc.term();
		}

		int stop(bool exit = true);
		int restore(const std::string& pathExe);

		int wait(double timeout);
protected:
	void eraseTemp();
private:
		cgutils::Subprocess proc;

		std::string tempdir;
		std::string routerTempFileIni;
		std::string logfile;
		std::string tracefile;

};

#endif /* ROUTER_H_ */
